##--------------------------------------------------------------------------------------------------------
## SCRIPT : Case studies
##
## Authors : Authier Matthieu
## Last update : 2019-10-09
##
## R version 3.5.2 (2018-07-02) -- "Eggshell Igloo"
## Copyright (C) 2018 The R Foundation for Statistical Computing
## Platform: x86_64-w64-mingw32/x64 (64-bit)
##--------------------------------------------------------------------------------------------------------

lapply(c("ggplot2", "ggthemes", "dplyr", "reshape2", "mvtnorm", "readxl", "hett", "arm"), 
       library, character.only = TRUE)

rm(list = ls())

setwd(WorkDir <- getwd()) # set path to your working directory
DataDir <- paste(WorkDir, "data", sep = "/")
OutDir <- paste(WorkDir, "output", sep = "/")

dolphins <- readxl::read_xls(paste(DataDir, "CetaceansCaseStudy.xls", sep = "/"), sheet = 1)
dolphins$N_hat <- as.numeric(dolphins$N_hat)
dolphins$se <- as.numeric(dolphins$se)
dolphins$cv <- as.numeric(dolphins$cv)
dolphins$lower <- as.numeric(dolphins$lower)
dolphins$upper <- as.numeric(dolphins$upper)

dolphins %>% 
  # filter(cv < 100) %>% 
  ggplot(aes(x = year, y = N_hat, group = Species_stock, color = Species_stock)) +
  geom_point() + geom_smooth(method = "lm", se = FALSE) +
  xlab("Year") + ylab("Abundance") +
  scale_y_log10() +
  theme_bw()

### trend analysis
regularized_trend <- function(df, sp) {
  df <- subset(df, Species_stock == sp)
  df$t <- with(df, (year - min(year))/diff(range(year)))
  df$p <- with(df, N_hat/N_hat[1])
  
  return(arm::bayesglm(log(p) ~ -1 + t, 
                       family = gaussian, 
                       data = df,
                       prior.mean = 0,
                       prior.scale = -log(2) * cos(pi * (0.025 - 0.5)) / sin(pi * (0.025 - 0.5)),
                       prior.df = 1,
                       epsilon = 1e-7, maxit = 500
                       ))
}

all_reg <- lapply(unique(dolphins$Species_stock), 
                  regularized_trend, df = dolphins#subset(dolphins, cv < 100)
                  )

### summary stat
sumstat <- do.call('rbind',
                   lapply(all_reg, function(armfit) { c(summary(armfit)$coefficients[-3], ifelse(armfit$converged, 1, 0), nrow(armfit$data), diff(range(armfit$data$year))) })
                   )
sumstat <- as.data.frame(sumstat)
names(sumstat) <- c("theta", "se", "pval", "conv", "n", "span")
sumstat$Species_stock <- unique(dolphins$Species_stock)

growth_rate <- function(df, sp) {
  get_summary <- function(x) {
    round(c(median(x), mad(x), as.numeric(coda::HPDinterval(coda::as.mcmc(x)), prob = 0.8)), 3)
  }
  df <- subset(df, Species_stock == sp)
  theta <- exp(sn::rst(1e6, xi = df$theta, omega = df$se, nu = df$n - 1))
  
  result <- as.data.frame(rbind(get_summary(theta), get_summary(theta^(1/df$span))))
  names(result) <- c("Median", "MAD", "lower", "upper")
  result$param <- c("total", "yearly")
  result$Species_stock <- sp
  return(result)
}

results <- do.call('rbind',
                   lapply(sumstat$Species_stock, growth_rate, df = sumstat)
                   )

results$Species_stock <- factor(results$Species_stock,
                                levels = subset(results, param == "yearly")$Species_stock[order(subset(results, param == "yearly")$Median)])

results$n <- sumstat$n[match(results$Species_stock, sumstat$Species_stock)]

theme_set(theme_bw(base_size = 12))

results %>% 
  filter(param == "yearly") %>% 
  ggplot(aes(x = Species_stock, y = Median)) +
  geom_hline(yintercept = 1, color = "red", linetype = "dotted") +
  geom_linerange(aes(x = Species_stock, ymin = lower, ymax = upper), color = "#b2df8a", size = 2) +
  geom_point(shape = 21, fill = "#b2df8a") +
  xlab("Species/stock") + ylab("Annual growth rate") +
  scale_y_continuous(breaks = seq(0.80, 1.30, 0.05)) +
  # facet_wrap(~ param, ncol = 1, scales = "free_x") +
  coord_flip() +
  theme(legend.position = "top",
        legend.text = element_text(size = 10),
        plot.title = element_text(lineheight = 0.8, face = "bold"),
        axis.text = element_text(size = 10),
        # panel.grid.major = element_blank(),
        # panel.grid.minor = element_blank(),
        panel.background = element_blank()
        )

ggsave(paste(OutDir, "casestudies.png", sep = "/"), dpi = 600,
       width = 15, height = 12, units = "cm")

results <- cbind(results,
                 sumstat[match(results$Species_stock, sumstat$Species_stock), c("span")]
                 )
names(results)[8] <- "span"
results %>% 
  filter(param == "yearly") %>% 
  ggplot(aes(x = n, y = Median)) +
  geom_point() +
  xlab("Sample size") + ylab("Annual growth rate") +
  scale_y_continuous(breaks = seq(0.80, 1.30, 0.05)) +
  theme_bw()

results %>% 
  filter(param == "yearly") %>% 
  ggplot(aes(x = span, y = Median)) +
  geom_point() +
  xlab("Study duration") + ylab("Annual growth rate") +
  scale_y_continuous(breaks = seq(0.80, 1.30, 0.05)) +
  theme_bw()

write.table(results,
            paste(OutDir, "casestudies.txt", sep = "/"), quote = FALSE,
            col.names = TRUE, row.names = FALSE, sep = "\t"
            )
