##--------------------------------------------------------------------------------------------------------
## SCRIPT : Case studies
##
## Authors : Authier Matthieu
## Last update : 2019-04-17
##
## R version 3.5.2 (2018-07-02) -- "Eggshell Igloo"
## Copyright (C) 2018 The R Foundation for Statistical Computing
## Platform: x86_64-w64-mingw32/x64 (64-bit)
##--------------------------------------------------------------------------------------------------------

lapply(c("ggplot2", "ggthemes", "dplyr", "reshape2", "mvtnorm", "readxl", "hett", "arm"), 
       library, character.only = TRUE)

rm(list = ls())

setwd(WorkDir <- getwd()) # set path to your working directory
DataDir <- paste(WorkDir, "data", sep = "/")
OutDir <- paste(WorkDir, "output", sep = "/")

dolphins <- readxl::read_xls(paste(DataDir, "CetaceansCaseStudy.xls", sep = "/"), sheet = 1)
dolphins$N_hat <- as.numeric(dolphins$N_hat)
dolphins$se <- as.numeric(dolphins$se)
dolphins$cv <- as.numeric(dolphins$cv)
dolphins$lower <- as.numeric(dolphins$lower)
dolphins$upper <- as.numeric(dolphins$upper)

n <- dolphins %>% 
  group_by(Species_stock) %>% 
  summarize(sample_size = n()) %>% 
  filter(sample_size > 3)

dolphins <- dolphins %>% 
  filter(Species_stock %in% n$Species_stock)

### trend analysis
regularized_trend <- function(df, sp, reg = TRUE, prior = "weak", rounding_decimal = 6) {
  get_summary <- function(x) {
    round(as.numeric(coda::HPDinterval(coda::as.mcmc(x)), prob = 0.8), 3)
  }
  
  df_sp <- subset(df, Species_stock == sp)
  
  inc_lm <- function(i) {
    df_i <- df_sp[1:i, ]
    df_i$t <- with(df_i, (year - min(year)) / diff(range(year)))
    df_i$p <- with(df_i, N_hat / N_hat[1])
    
    if(reg) {
      if(prior == "weak") {
        prior_scale <- -log(2) * cos(pi * (0.025 - 0.5)) / sin(pi * (0.025 - 0.5))
        prior_df <- 1
      }
      else {
        prior_scale <- log(2) / 2
        prior_df <- Inf
      }
      armfit <- arm::bayesglm(log(p) ~ -1 + t, 
                              family = gaussian, 
                              data = df_i,
                              prior.mean = 0,
                              prior.scale = prior_scale,
                              prior.df = prior_df,
                              epsilon = 1e-7, maxit = 500
                              )
    }
    
    else {
      armfit <- glm(log(p) ~ -1 + t, 
                    family = gaussian, 
                    data = df_i,
                    epsilon = 1e-7, maxit = 500
                    )
    }
    
    theta <- exp(sn::rst(1e6, 
                         xi = summary(armfit)$coefficients[1],
                         omega = summary(armfit)$coefficients[2], 
                         nu = nrow(armfit$data) - 1
                         )
                 )
    
    theta <- theta^(1 / diff(range(armfit$data$year)))
    
    estimate <- data.frame(theta = round(median(theta), rounding_decimal), 
                           se = round(mad(theta), rounding_decimal),
                           lower = get_summary(theta)[1],
                           upper = get_summary(theta)[2],
                           conv = ifelse(armfit$converged, 1, 0), 
                           n = nrow(armfit$data), 
                           span = diff(range(armfit$data$year))
                           )
    return(estimate)
  }
  all_reg <- do.call("rbind", lapply(3:nrow(df_sp), inc_lm))
  all_reg$rm <- 3:nrow(df_sp)
  all_reg$species <- sp
  
  return(all_reg)
}

all_reg <- lapply(unique(dolphins$Species_stock), 
                  regularized_trend, df = dolphins
                  )

### summary stat
sumstat <- do.call('rbind', all_reg)
sumstat$reg <- "weakly\ninformative"

###
all_unreg <- lapply(unique(dolphins$Species_stock), 
                    regularized_trend, df = dolphins, reg = FALSE
                    )
sumstat2 <- do.call('rbind', all_unreg)
sumstat2$reg <- "unregularized"

all_streg <- lapply(unique(dolphins$Species_stock), 
                    regularized_trend, df = dolphins, prior = "strong"
                    )
sumstat3 <- do.call('rbind', all_streg)
sumstat3$reg <- "informative"


### order species with sample size
lev <- n$Species_stock[order(n$sample_size)]

theme_set(theme_bw(base_size = 12))
rbind(sumstat, sumstat2, sumstat3) %>% 
  mutate(species_fac = factor(species, levels = lev),
         reg = factor(reg, levels = c("unregularized", "informative", "weakly\ninformative"))
         ) %>%
  ggplot(aes(x = rm, y = theta, ymin = lower, ymax = upper, group = reg, color = reg)) +
  geom_hline(yintercept = 1, color = "red", linetype = "dotted") +
  geom_linerange(position = position_dodge(width = 0.8)) +
  geom_point(position = position_dodge(width = 0.8)) +
  geom_line(position = position_dodge(width = 0.8)) +
  scale_x_continuous(name = "Sample size", breaks = c(3:13)) + 
  ylab("Annual growth rate") +
  scale_color_manual(name = "", values = c("#a6cee3", "#1f78b4", "#b2df8a")) +
  facet_wrap(~ species_fac, ncol = 3, scales = "free_y") +
  # coord_flip() +
  theme(legend.position = "top",
        legend.text = element_text(size = 10),
        plot.title = element_text(lineheight = 0.8, face = "bold"),
        axis.text = element_text(size = 10),
        # panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.background = element_blank()
        )
ggsave(paste(OutDir, "casestudies_stability.png", sep = "/"), dpi = 600,
       width = 20, height = 16, units = "cm")